# -*- coding: utf-8; -*-
################################################################################
#
#  wuttaweb -- Web App for Wutta Framework
#  Copyright © 2024 Lance Edgar
#
#  This file is part of Wutta Framework.
#
#  Wutta Framework is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option) any
#  later version.
#
#  Wutta Framework is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#  more details.
#
#  You should have received a copy of the GNU General Public License along with
#  Wutta Framework.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Upgrade Views
"""

from sqlalchemy import orm

from wuttjamaican.db.model import Upgrade
from wuttaweb.views import MasterView
from wuttaweb.forms import widgets
from wuttaweb.forms.schema import UserRef, WuttaEnum


class UpgradeView(MasterView):
    """
    Master view for upgrades.

    Default route prefix is ``upgrades``.

    Notable URLs provided by this class:

    * ``/upgrades/``
    * ``/upgrades/new``
    * ``/upgrades/XXX``
    * ``/upgrades/XXX/edit``
    * ``/upgrades/XXX/delete``
    """
    model_class = Upgrade

    grid_columns = [
        'created',
        'description',
        'status',
        'executed',
        'executed_by',
    ]

    sort_defaults = ('created', 'desc')

    def configure_grid(self, g):
        """ """
        super().configure_grid(g)
        model = self.app.model
        enum = self.app.enum

        # description
        g.set_link('description')

        # created
        g.set_renderer('created', self.grid_render_datetime)

        # created_by
        g.set_link('created_by')
        Creator = orm.aliased(model.User)
        g.set_joiner('created_by', lambda q: q.join(Creator,
                                                    Creator.uuid == model.Upgrade.created_by_uuid))
        g.set_filter('created_by', Creator.username,
                     label="Created By Username")

        # status
        g.set_renderer('status', self.grid_render_enum, enum=enum.UpgradeStatus)

        # executed_by
        g.set_link('executed_by')
        Executor = orm.aliased(model.User)
        g.set_joiner('executed_by', lambda q: q.outerjoin(Executor,
                                                          Executor.uuid == model.Upgrade.executed_by_uuid))
        g.set_filter('executed_by', Executor.username,
                     label="Executed By Username")

    def grid_row_class(self, upgrade, data, i):
        """ """
        enum = self.app.enum
        if upgrade.status == enum.UpgradeStatus.EXECUTING:
            return 'has-background-warning'
        if upgrade.status == enum.UpgradeStatus.FAILURE:
            return 'has-background-warning'

    def configure_form(self, f):
        """ """
        super().configure_form(f)
        enum = self.app.enum
        upgrade = f.model_instance

        # never show these
        f.remove('created_by_uuid',
                 'executing',
                 'executed_by_uuid')

        # sequence sanity
        f.fields.set_sequence([
            'description',
            'notes',
            'status',
            'created',
            'created_by',
            'executed',
            'executed_by',
        ])

        # created
        if self.creating or self.editing:
            f.remove('created')

        # created_by
        if self.creating or self.editing:
            f.remove('created_by')
        else:
            f.set_node('created_by', UserRef(self.request))

        # notes
        f.set_widget('notes', widgets.NotesWidget())

        # status
        if self.creating:
            f.remove('status')
        else:
            f.set_node('status', WuttaEnum(self.request, enum.UpgradeStatus))

        # exit_code
        if self.creating or not upgrade.executed:
            f.remove('exit_code')

        # executed
        if self.creating or self.editing or not upgrade.executed:
            f.remove('executed')

        # executed_by
        if self.creating or self.editing or not upgrade.executed:
            f.remove('executed_by')
        else:
            f.set_node('executed_by', UserRef(self.request))

    def objectify(self, form):
        """ """
        upgrade = super().objectify(form)
        enum = self.app.enum

        # set user, status when creating
        if self.creating:
            upgrade.created_by = self.request.user
            upgrade.status = enum.UpgradeStatus.PENDING

        return upgrade

    @classmethod
    def defaults(cls, config):
        """ """

        # nb. Upgrade may come from custom model
        wutta_config = config.registry.settings['wutta_config']
        app = wutta_config.get_app()
        cls.model_class = app.model.Upgrade

        cls._defaults(config)


def defaults(config, **kwargs):
    base = globals()

    UpgradeView = kwargs.get('UpgradeView', base['UpgradeView'])
    UpgradeView.defaults(config)


def includeme(config):
    defaults(config)
